package org.zen.xhtml.editor;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.texteditor.ResourceAction;
import org.eclipse.ui.texteditor.TextOperationAction;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.zen.xhtml.views.XHTMLContentOutlinePage;
import org.zen.xhtml.editor.preferencePage.IPreferenceConstants;
import org.zen.xhtml.rulers.HorizontalRuler;
import org.zen.xhtml.rulers.VerticalRuler;

/**
 * @author Zenil
 * @version 1.0
 */
public class XHTMLEditor extends TextEditor {

	/** The color manager */
	private ColorManager colorManager;
	
	private VerticalRuler fVerticalRuler;
	
	/** Outine Page */
	private XHTMLContentOutlinePage fOutlinePage;
	
	private HorizontalRuler fHRuler;
	
	private Composite fMainComposite;
	
	/**
	 * Constructor for XHTMLEditor.
	 */
	public XHTMLEditor() {
		super();
		colorManager = ColorManager.getColorManager();
		setSourceViewerConfiguration(new XHTMLSourceViewerConfiguration(colorManager));
		setDocumentProvider(new XHTMLDocumentProvider());
		fVerticalRuler = new VerticalRuler(VERTICAL_RULER_WIDTH);
		
		setPreferenceStore(XHTMLEditorPlugin.getDefault().getPreferenceStore());
	}
	
	public void doSetInput(IEditorInput input) throws CoreException {
		super.doSetInput(input);
		if (fOutlinePage != null)
			fOutlinePage.setInput(input);
	}
	
	public void doRevertToSaved() {
		super.doRevertToSaved();
		if (fOutlinePage != null)
			fOutlinePage.update();
	}
	
	public void doSave(IProgressMonitor monitor) {
		super.doSave(monitor);
		if (fOutlinePage != null)
			fOutlinePage.update();
	}
	
	public void doSaveAs() {
		super.doSaveAs();
		if (fOutlinePage != null)
			fOutlinePage.update();
	}
	
	/**
	 * @see org.eclipse.ui.texteditor.StatusTextEditor#createPartControl(Composite)
	 */
	public void createPartControl(Composite parent) {

		super.createPartControl(parent);
		
		Composite childComp =(Composite) ((Composite)parent.getChildren()[0]).getChildren()[0];
		GridLayout layout = new GridLayout();
		layout.marginHeight=0;
		layout.marginWidth=0;
		layout.verticalSpacing=2;
		childComp.setLayout(layout);

		GridData data = new GridData(GridData.FILL_BOTH);
		childComp.getChildren()[0].setLayoutData(data);
		
		fMainComposite = childComp;
		
		if(isHorizontalRulerVisible()){
			createHorizontalRuler(fMainComposite);
		}		
	}

		
	/**
	 * This method is called from the createPartControl() of AbstractTextEditor
	 * The createAction() in AbstractTextEditor creates this editor's standard actions and connects 
	 * them with the global workbench actions. These actions include undo,redo,cut,copy,
	 * paste,gotoline,save,revert to saved,addbookmark,addtask,print,delete. These are all
	 * various types of TextEditorActions.
	 * The createAction() in TextEditor creates 3 actions - convert line delims in windows,unix,macs
	 * Creating two extra actions for content assistance
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#createActions()
	 */
	protected void createActions() {
		
		super.createActions();
		
		ResourceAction action =   new TextOperationAction(XHTMLEditorPlugin.getDefault().getResourceBundle(), "ContentAssistProposal.", this, ISourceViewer.CONTENTASSIST_PROPOSALS);//$NON-NLS-1$
		/* Set the definitionID and help id if required ! */
		//action.setActionDefinitionId("");
		//action.setHelpContextId("");
		setAction(IXHTMLEditorConstants.ACTION_CONTENT_ASSIST_PROPOSAL,action);
		
		action =   new TextOperationAction(XHTMLEditorPlugin.getDefault().getResourceBundle(), "ContentAssistTip.", this, ISourceViewer.CONTENTASSIST_CONTEXT_INFORMATION);//$NON-NLS-1$
		setAction(IXHTMLEditorConstants.ACTION_CONTENT_ASSIST_TIP,action);
		
		action= new TextOperationAction(XHTMLEditorPlugin.getDefault().getResourceBundle(), "ShowHTMLInfo.", this, ISourceViewer.INFORMATION); //$NON-NLS-1$
		setAction(IXHTMLEditorConstants.ACTION_HTML_INFO, action);
		
	}
	
	/**
	 * The super class AbstractTextEditor method adds cut,copy,paste actions
	 * The super class TextEditor method adds shift left,shift right actions
	 * to the editor context menu. Adding some more items here
	 * Adding two actions for content assistance
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#editorContextMenuAboutToShow(IMenuManager)
	 */
	protected void editorContextMenuAboutToShow(IMenuManager parentMenu){
		
		super.editorContextMenuAboutToShow(parentMenu);
		addAction(parentMenu, IXHTMLEditorConstants.ACTION_CONTENT_ASSIST_PROPOSAL);
		addAction(parentMenu, IXHTMLEditorConstants.ACTION_CONTENT_ASSIST_TIP);
				
		/** REFER ReadMeEditor Code For other ways of putting actions!!! */		
	}
	
	
	/*
	 * @see AbstractTextEditor#createVerticalRuler()
	 */
	protected IVerticalRuler createVerticalRuler() {
		return fVerticalRuler.createVerticalRuler();
	}
	
	/**
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#handlePreferenceStoreChanged(PropertyChangeEvent)
	 */
	protected void handlePreferenceStoreChanged(PropertyChangeEvent event) {
		super.handlePreferenceStoreChanged(event);
		ISourceViewer sourceViewer= getSourceViewer();
			if (sourceViewer == null)
				return;
				
		String property= event.getProperty();	
		if(property.equals(IPreferenceConstants.VRULER_PREFERENCE)) {
			if(fVerticalRuler.isLineNumberRulerVisible()) {
				fVerticalRuler.showLineNumberRuler();
			} else {
				fVerticalRuler.hideLineNumberRuler ();
			}
		}
		
		if(property.equals(IPreferenceConstants.HRULER_PREFERENCE)) {
			if(isHorizontalRulerVisible()) {
				createHorizontalRuler(fMainComposite);
			} else {
				removeHorizontalRuler(fMainComposite);
			}
			if(fMainComposite !=null && !fMainComposite.isDisposed()) {
				fMainComposite.layout(true);
			}
		}
		
		if (PREFERENCE_FONT.equals(property)) {
			fHRuler.setRedraw(false);
			fHRuler.setFont(getSourceViewer().getTextWidget().getFont());
			fHRuler.setRedraw(true);
		}	
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		if (fOutlinePage != null)
			fOutlinePage.setInput(null);
		super.dispose();
	}	
	
	public Object getAdapter(Class required) {
		if (IContentOutlinePage.class.equals(required)) {
			if (fOutlinePage == null) {
				fOutlinePage= new XHTMLContentOutlinePage(getDocumentProvider(), this);
				if (getEditorInput() != null)
					fOutlinePage.setInput(getEditorInput());
			}
			return fOutlinePage;
		}
		return super.getAdapter(required);
	}
	
	private boolean isHorizontalRulerVisible() {
		IPreferenceStore store = XHTMLEditorPlugin.getDefault().getPreferenceStore();
		return store.getBoolean(IPreferenceConstants.HRULER_PREFERENCE);
	}
	
	private void createHorizontalRuler(Composite parent) {
		GC gc = new GC(getSourceViewer().getTextWidget());
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.heightHint=gc.getFontMetrics().getHeight();
		gc.dispose();

		
		fHRuler = new HorizontalRuler(parent,fVerticalRuler);
		fHRuler.setFont(getSourceViewer().getTextWidget().getFont());
		fHRuler.setSourceViewer(getSourceViewer());
		fHRuler.setLayoutData(data);
		fHRuler.moveAbove(null);
	}	
	
	private void removeHorizontalRuler(Composite parent){
		if(fHRuler!=null && !fHRuler.isDisposed()) {
			fHRuler.dispose();
		}		
	}
}
