package org.zen.xhtml.editor;

import javax.swing.text.html.HTML;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.DefaultInformationControl;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IInformationControl;
import org.eclipse.jface.text.IInformationControlCreator;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextHover;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.contentassist.ContentAssistant;
import org.eclipse.jface.text.contentassist.IContentAssistant;
import org.eclipse.jface.text.information.IInformationPresenter;
import org.eclipse.jface.text.information.IInformationProvider;
import org.eclipse.jface.text.information.InformationPresenter;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.zen.xhtml.editor.contentassist.XHTMLContentAssistProcessor;
import org.zen.xhtml.editor.scanner.XHTMLTagScanner;
import org.zen.xhtml.editor.util.XHTMLPrinter;
import org.zen.xhtml.editor.util.XHTMLWordFinder;
import org.zen.xhtml.editor.util.presenter.HTMLTextPresenter;

/**
 * @author Zenil
 * @version 1.0
 */
public class XHTMLSourceViewerConfiguration extends SourceViewerConfiguration {

	private RuleBasedScanner fTagScanner, fDefaultScanner, fCommentScanner, fPIScanner;

	private ITextHover fTextHover;

	private ColorManager colorManager;

	public XHTMLSourceViewerConfiguration(ColorManager colorManager) {
		this.colorManager = colorManager;
	}

	/**
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getConfiguredContentTypes(ISourceViewer)
	 */
	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		return new String[] {
			IDocument.DEFAULT_CONTENT_TYPE,
			IXHTMLEditorConstants.CONTENT_TYPE_HTML_COMMENT,
			IXHTMLEditorConstants.CONTENT_TYPE_HTML_TAG,
			IXHTMLEditorConstants.CONTENT_TYPE_HTML_PI };
	}

	/**
	 * Scanner for content type HTML_TAG
	 * @return RuleBasedScanner
	 */
	protected RuleBasedScanner getHTMLTagScanner() {
		if (fTagScanner == null) {
			fTagScanner = new XHTMLTagScanner(colorManager);
		}
		return fTagScanner;
	}

	/**
	 * Scanner for content type default
	 * @return RuleBasedScanner
	 */
	protected RuleBasedScanner getHTMLDefaultScanner() {
		if (fDefaultScanner == null) {
			IToken tagAttribute = new Token(new TextAttribute(colorManager.getColor(IXHTMLColorConstants.DEFAULT)));
			fDefaultScanner = new RuleBasedScanner();
			fDefaultScanner.setDefaultReturnToken(tagAttribute);
		}
		return fDefaultScanner;
	}

	/**
	 * Scanner for content type HTML_COMMENT.i.e whenever a proper sequence of
	 * '<!--....-->' is encountered  this scanner is invoked.
	 * @return RuleBasedScanner
	 */
	protected RuleBasedScanner getHTMLCommentScanner() {
		if (fCommentScanner == null) {
			IToken tagAttribute = new Token(new TextAttribute(colorManager.getColor(IXHTMLColorConstants.HTML_COMMENT)));
			fCommentScanner = new RuleBasedScanner();
			fCommentScanner.setDefaultReturnToken(tagAttribute);
		}
		return fCommentScanner;
	}

	/**
	 * Scanner for content type CONTENT_TYPE_HTML_PI.i.e whenever a proper sequence of
	 * '<?....?>' is encountered  this scanner is invoked.
	 * @return RuleBasedScanner
	 */
	protected RuleBasedScanner getHTMLPIScanner() {
		if (fPIScanner == null) {
			IToken tagAttribute = new Token(new TextAttribute(colorManager.getColor(IXHTMLColorConstants.PROC_INSTR)));
			fPIScanner = new RuleBasedScanner();
			fPIScanner.setDefaultReturnToken(tagAttribute);
		}
		return fPIScanner;
	}

	/**
	 * Setting the damager repairer.Since there are basically three regions/partitions
	 * there are three corresponding scanners.The comment scanner,the default scanner,
	 * and the tag scanner. Each scanner has a default attribute which contains
	 * the presentation details for the region/partition.
	 * When the damager encounters a damage to a particular region/partition/contenttype
	 * it calls the corresponding repairer which uses the appropriate scanner
	 * 
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getPresentationReconciler(ISourceViewer)
	 */
	public IPresentationReconciler getPresentationReconciler(ISourceViewer sourceViewer) {
		PresentationReconciler reconciler = new PresentationReconciler();

		DefaultDamagerRepairer dr = new DefaultDamagerRepairer(getHTMLTagScanner());
		reconciler.setDamager(dr, IXHTMLEditorConstants.CONTENT_TYPE_HTML_TAG);
		reconciler.setRepairer(dr, IXHTMLEditorConstants.CONTENT_TYPE_HTML_TAG);

		dr = new DefaultDamagerRepairer(getHTMLDefaultScanner());
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);

		dr = new DefaultDamagerRepairer(getHTMLCommentScanner());
		reconciler.setDamager(dr, IXHTMLEditorConstants.CONTENT_TYPE_HTML_COMMENT);
		reconciler.setRepairer(dr, IXHTMLEditorConstants.CONTENT_TYPE_HTML_COMMENT);

		dr = new DefaultDamagerRepairer(getHTMLPIScanner());
		reconciler.setDamager(dr, IXHTMLEditorConstants.CONTENT_TYPE_HTML_PI);
		reconciler.setRepairer(dr, IXHTMLEditorConstants.CONTENT_TYPE_HTML_PI);

		return reconciler;
	}

	/**
	 * Is invoked whenever hovering happens.
	 * (Inorder for TextHovering to be invoked getInformationControlCreator() 
	 * should also  be defined. By default it is defined in the super class .
	 * Therefore we dont have to define it here.). The getInformationControlCreator()
	 * returns a DefaultInformationControl().This control is used to show the hover 
	 * information. If we want to show hover information in our own spl. control
	 * ,we have to re-define our own getInformationControlCreator()..
	 * 
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getTextHover(ISourceViewer, String)
	 */
	public ITextHover getTextHover(ISourceViewer sourceViewer, String contentType) {
		if (fTextHover == null) {
			fTextHover = new XHTMLTextHover();
		}
		return fTextHover;
	}

	/**
	 * This presenter is used by the TextOperationAction with id ISourceViewer.INFORMATION.
	 * If this action is defined, when it is executed(by pressing F2) ,the following
	 * presenter will be invoked to construct and show the informaton.
	 * This presenter has no relation to Hovering(which has its own presenter,which 
	 * it gets from getInformationControlCreator())
	 * This presenter will only be used only when user presses F2 or
	 * HTMLInfo action is invoked(Check XHTMLEditor for HTMLInfo action)
	 * 
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getInformationPresenter(ISourceViewer)
	 */
	public IInformationPresenter getInformationPresenter(ISourceViewer sourceViewer) {
		InformationPresenter presenter = new InformationPresenter(getInformationControlCreator(sourceViewer, true));
		IInformationProvider provider = new IInformationProvider() {
			public String getInformation(ITextViewer textViewer, IRegion subject) {
				try {
					if (subject.getLength() > -1) {
						String s = textViewer.getDocument().get(subject.getOffset(), subject.getLength());
						if (s != null && s.length() > 0) {
							s = getHTMLFormattedText(s);
							return s;
						}						
					}
				} catch (BadLocationException x) {
				}
				return null;
			}
			public IRegion getSubject(ITextViewer textViewer, int offset) {
				return XHTMLWordFinder.findWord(textViewer.getDocument(), offset);
			}

		};
		presenter.setInformationProvider(provider, IDocument.DEFAULT_CONTENT_TYPE);
		presenter.setInformationProvider(provider, IXHTMLEditorConstants.CONTENT_TYPE_HTML_TAG);
		presenter.setInformationProvider(provider, IXHTMLEditorConstants.CONTENT_TYPE_HTML_PI);
		presenter.setInformationProvider(provider, IXHTMLEditorConstants.CONTENT_TYPE_HTML_COMMENT);
		presenter.setSizeConstraints(60, 10, true, true);
		return presenter;
	}

	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public IContentAssistant getContentAssistant(ISourceViewer sourceViewer) {

		/*
		 * The ContentAssistant class is responsible for showing the completion
		 * proposals and context informations. It has CompletionProposalPopup
		 * and ContextInformationPopup ,controls which will show these information.
		 */
		ContentAssistant assistant = new ContentAssistant();
		assistant.setContentAssistProcessor(new XHTMLContentAssistProcessor(), IXHTMLEditorConstants.CONTENT_TYPE_HTML_TAG);
		assistant.enableAutoActivation(true);
		assistant.setAutoActivationDelay(500);
		assistant.setProposalPopupOrientation(assistant.PROPOSAL_OVERLAY);
		assistant.setContextInformationPopupOrientation(assistant.CONTEXT_INFO_ABOVE);
		assistant.setContextInformationPopupBackground(colorManager.getColor(IXHTMLColorConstants.CONTENT_ASSIST_COLOR));

		//This is required for showing additional proposal info.
		//A proposal can have a context information and might have additional proposal 
		//info.These additonal proposal infos will be shown using this InformationControl
		assistant.setInformationControlCreator(getInformationControlCreator(sourceViewer));

		return assistant;
	}

	/**
	 * Returns the information control creator. The creator is a factory creating information
	 * controls for the given source viewer. This implementation always returns a creator for
	 * <code>DefaultInformationControl</code> instances.
	 * 
	 * @param sourceViewer the source viewer to be configured by this configuration
	 * @return the information control creator or <code>null</code> if no information support should be installed
	 * @since 2.0
	 */
	public IInformationControlCreator getInformationControlCreator(ISourceViewer sourceViewer) {
		return getInformationControlCreator(sourceViewer, false);
	}

	private IInformationControlCreator getInformationControlCreator(ISourceViewer sourceViewer, boolean vhBar) {
		final int style = vhBar ? SWT.V_SCROLL | SWT.H_SCROLL : SWT.NONE;
		return new IInformationControlCreator() {
			public IInformationControl createInformationControl(Shell parent) {
				return new DefaultInformationControl(parent, style, new HTMLTextPresenter(true));
			}
		};

	}

	private String getHTMLFormattedText(String text) {

		StringBuffer buff = new StringBuffer();
		HTML.Tag tag = HTML.getTag(text);
		if (tag != null) {
			XHTMLPrinter.addPageProlog(buff);
			XHTMLPrinter.addSmallHeader(buff, "This is an HTML Tag");
			XHTMLPrinter.addSmallHeader(buff, "Properties :");
			XHTMLPrinter.startBulletList(buff);
			XHTMLPrinter.addBullet(buff, "Name = " + text);
			XHTMLPrinter.addBullet(buff, "Block = " + tag.isBlock());
			XHTMLPrinter.addBullet(buff, "PreFormatted = " + tag.isPreformatted());
			XHTMLPrinter.addBullet(buff, "Breaks Flow = " + tag.breaksFlow());
			XHTMLPrinter.endBulletList(buff);
			XHTMLPrinter.addPageEpilog(buff);
			return buff.toString();
		}
		HTML.Attribute attr = HTML.getAttributeKey(text);
		if (attr != null) {
			XHTMLPrinter.insertPageProlog(buff, 0);
			XHTMLPrinter.addSmallHeader(buff, "This is an HTML Attribute");
			XHTMLPrinter.addSmallHeader(buff, "Properties :");
			XHTMLPrinter.startBulletList(buff);
			XHTMLPrinter.addBullet(buff, "Name = " + text);
			XHTMLPrinter.endBulletList(buff);
			XHTMLPrinter.addPageEpilog(buff);
			return buff.toString();
		}
		XHTMLPrinter.addPageProlog(buff);
		XHTMLPrinter.addSmallHeader(buff, text);
		XHTMLPrinter.addPageEpilog(buff);
		return buff.toString();
	}

	/* NOTE:
	 * For Writing your own DoubleClick strategy check out XMLEditor example
	 * For most purposes the default double click strategy is more than enough
	 */

}