package org.zen.xhtml.editor.contentassist;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.ICompletionProposalExtension;
import org.eclipse.jface.text.contentassist.IContextInformation;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;

/**
 * Special Completion proposal class that wraps a CompletionProposal object.
 * Delegates all the work to the default completion proposal object.
 * Also identifies whether a completion proposal is valid or not for the current
 * position
 * 
 * @author Zenil
 * @version 1.0
 *
 */
/* 
 * Ideally would have liked to extend CompletionProposal rather than wrap
 * it. But CompletionProposal is final class. 
 */
public class XHTMLCompletionProposal implements ICompletionProposal, ICompletionProposalExtension {

	private String fReplacementString;
	private int fReplacementOffset;
	private ICompletionProposal fProposal;
	private IContextInformation fContextInformation;
	
	/**
	 * Method XHTMLCompletionProposal.
	 * @param proposal
	 * @param replacementOffset This is the ofset of the position in the document
	 * where content assistance was invoked.Any replacement of text will start from
	 * this position/offset
	 */
	public XHTMLCompletionProposal(ICompletionProposal proposal,int replacementOffset) {
		fProposal = proposal;
		String displayString = fProposal.getDisplayString();
		//Assuming that replacementString and DisplayString are same!
		fReplacementString = displayString;
		fContextInformation = fProposal.getContextInformation();
		fReplacementOffset = replacementOffset;
	}
	
	/**
	 * @see org.eclipse.jface.text.contentassist.ICompletionProposal#apply(IDocument)
	 */
	public void apply(IDocument document) {
		//fProposal.apply(document);
		try {
			replace(document,0,fReplacementString.length(),fReplacementString 	);
		} catch (BadLocationException e) {
		}
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.ICompletionProposal#getSelection(IDocument)
	 */
	public Point getSelection(IDocument document) {
		return fProposal.getSelection(document);
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.ICompletionProposal#getAdditionalProposalInfo()
	 */
	public String getAdditionalProposalInfo() {
		return fProposal.getAdditionalProposalInfo();
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.ICompletionProposal#getDisplayString()
	 */
	public String getDisplayString() {
		return fProposal.getDisplayString();
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.ICompletionProposal#getImage()
	 */
	public Image getImage() {
		return fProposal.getImage();
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.ICompletionProposal#getContextInformation()
	 */
	public IContextInformation getContextInformation() {
		return fProposal.getContextInformation();
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.ICompletionProposalExtension#apply(IDocument, char, int)
	 */
	public void apply(IDocument document, char trigger, int offset) {
		try {
			int replacementLength = 0;
			
			//Compute the replacement length.
			//eg <press CTRL+SPACE here>For<After cursor reaches here>
			//fReplacementOffset = 87,offset=90 then
			//replacementLength is calculated as 3.
			//if proposal is 'Form', 'For' is replaced with Form
			int delta= offset - (fReplacementOffset + replacementLength);
			if (delta > 0)
				replacementLength += delta;			
				replace(document, fReplacementOffset, replacementLength, fReplacementString);			
		} catch (BadLocationException x) {
			// ignore
		}	
	}

	/**
	 * @param document
	 * @param offset The current cursor position.
	 * @see org.eclipse.jface.text.contentassist.ICompletionProposalExtension#isValidFor(IDocument, int)
	 */
	/*
	 * After the completon proposal pop up is shown,The CompletionProposalPopup class
	 * invokes this method to filter the proposals progressively as the user
	 * moves the cursor left or right.The offset represents the current cursor
	 * positon
	 */
	public boolean isValidFor(IDocument document, int offset) {
		if (offset < fReplacementOffset)
			return false;
	
		String word=fReplacementString;
		
		int wordLength= word == null ? 0 : word.length();
		if (offset >  fReplacementOffset + wordLength)
			return false;
		
		try {
			int length= offset - fReplacementOffset;
			String start= document.get(fReplacementOffset, length);
			return word.substring(0, length).equalsIgnoreCase(start);
		} catch (BadLocationException x) {
		}		
		
		return false;
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.ICompletionProposalExtension#getTriggerCharacters()
	 */
	public char[] getTriggerCharacters() {
		return null;
	}

	/**
	 * The context information for this proposal will be shown at this
	 * position
	 * @see org.eclipse.jface.text.contentassist.ICompletionProposalExtension#getContextInformationPosition()
	 */
	public int getContextInformationPosition() {
		return fReplacementOffset+fReplacementString.length();
	}
	
	private void replace(IDocument document, int offset, int length, String string) throws BadLocationException {
		//No need to replace the string if the replaced string is same as the
		//replacement string!
		if (!document.get(offset, length).equalsIgnoreCase(string)) {
			document.replace(offset, length, string);
		}
	}

}
